/*
   Qapla'! Quad‑led Klingon LED battle‑formation sketch.

   ➤ Ring 1 (LED_PIN1, SWITCH_PIN1)
       • While pressed  – two blue leaders followed by one red tail orbit.
       • While released – all LEDs steady green.

   ➤ Ring 2 (LED_PIN2, SWITCH_PIN2)
       • While pressed  – two red LEDs orbit like twin disruptor bolts.
       • While released – LEDs off (dark).

   ➤ Ring 3 (LED_PIN3, SWITCH_PIN3)
       • While pressed  – LEDs off (dark).
       • While released – chaotic yellow flicker across all LEDs (random brightness each frame).

   ➤ Ring 4 (LED_PIN4, SWITCH_PIN4)
       • While pressed  – flashes red 15 × (neon‑tube style), then holds solid red with a brief dim‑bright pulse.
       • While released – LEDs off.
*/

#include <Adafruit_NeoPixel.h>

// ---------- Hardware pins & sizes ----------
#define LED_PIN1     6   // Data pin for ring 1
#define LED_PIN2     7   // Data pin for ring 2
#define LED_PIN3     8   // Data pin for ring 3
#define LED_PIN4     9   // Data pin for ring 4

#define LED_COUNT1   9
#define LED_COUNT2   9
#define LED_COUNT3   9
#define LED_COUNT4   9

#define SWITCH_PIN1  2   // Micro‑switch for ring 1 (active‑LOW)
#define SWITCH_PIN2  3   // Micro‑switch for ring 2 (active‑LOW)
#define SWITCH_PIN3  4   // Micro‑switch for ring 3 (active‑LOW)
#define SWITCH_PIN4  5   // Micro‑switch for ring 4 (active‑LOW)

// ---------- Strip objects ----------
Adafruit_NeoPixel ring1(LED_COUNT1, LED_PIN1, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel ring2(LED_COUNT2, LED_PIN2, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel ring3(LED_COUNT3, LED_PIN3, NEO_GRB + NEO_KHZ800);
Adafruit_NeoPixel ring4(LED_COUNT4, LED_PIN4, NEO_GRB + NEO_KHZ800);

// ---------- Colors ----------
const uint32_t COLOR_RED   = ring1.Color(255, 0,   0);
const uint32_t COLOR_BLUE  = ring1.Color(0,   0, 255);
const uint32_t COLOR_GREEN = ring1.Color(0, 255,   0);
// Yellow is created dynamically (varying brightness)

// ---------- Timing ----------
const uint8_t  SPEED_MS = 120;  // Frame delay for shared animations (ms)

// ===== Ring 4 animation parameters =====
// Modify these to alter flash count & fade behaviour.
const uint8_t  BLINKS_RING4   = 15;   // Number of flashes before steady red
const uint16_t BLINK_ON_MS    = 60;   // ON‑time for each flash (ms)
const uint16_t BLINK_OFF_MS   = 60;   // OFF‑time for each flash (ms)
const uint8_t  R4_BRIGHT_LVL  = 255;  // Brightness of solid red (0‑255)
const uint8_t  R4_DIM_LVL     = 80;   // Intermediate dim level (0‑255)
const uint16_t DIM_DELAY_MS   = 150;  // How long the dim state lasts (ms)
// ---------------------------------------

void setup() {
  ring1.begin();
  ring2.begin();
  ring3.begin();
  ring4.begin();

  ring1.show();
  ring2.show();
  ring3.show();
  ring4.show();

  pinMode(SWITCH_PIN1, INPUT_PULLUP);
  pinMode(SWITCH_PIN2, INPUT_PULLUP);
  pinMode(SWITCH_PIN3, INPUT_PULLUP);
  pinMode(SWITCH_PIN4, INPUT_PULLUP);

  // Seed RNG for chaotic flicker (Ring 3)
  randomSeed(analogRead(A0));
}

void loop() {
  static uint8_t head1 = 0;  // Orbit index for Ring 1
  static uint8_t head2 = 0;  // Orbit index for Ring 2

  // ---------- Ring 1 logic (blue‑blue‑red chase) ----------
  if (digitalRead(SWITCH_PIN1) == LOW) {
    ring1.clear();
    ring1.setPixelColor(head1 % LED_COUNT1, COLOR_BLUE);
    ring1.setPixelColor((head1 + 1) % LED_COUNT1, COLOR_BLUE);
    ring1.setPixelColor((head1 + 2) % LED_COUNT1, COLOR_RED);
    ring1.show();
    head1 = (head1 + 1) % LED_COUNT1;
  } else {
    for (uint8_t i = 0; i < LED_COUNT1; i++) {
      ring1.setPixelColor(i, COLOR_GREEN);
    }
    ring1.show();
  }

  // ---------- Ring 2 logic (dual‑red chase) ----------
  if (digitalRead(SWITCH_PIN2) == LOW) {
    ring2.clear();
    ring2.setPixelColor(head2 % LED_COUNT2, COLOR_RED);
    ring2.setPixelColor((head2 + 1) % LED_COUNT2, COLOR_RED);
    ring2.show();
    head2 = (head2 + 1) % LED_COUNT2;
  } else {
    ring2.clear();
    ring2.show();
  }

  // ---------- Ring 3 logic (chaotic yellow flicker) ----------
  if (digitalRead(SWITCH_PIN3) == HIGH) { // Flicker when switch is RELEASED
    for (uint8_t i = 0; i < LED_COUNT3; i++) {
      uint8_t b = random(60, 256);  // Random brightness 60‑255
      ring3.setPixelColor(i, ring3.Color(b, b, 0)); // Yellow with varying intensity
    }
    ring3.show();
  } else {
    ring3.clear();
    ring3.show();
  }

  // ---------- Ring 4 logic (15 flashes → dim → bright) ----------
  static bool seqComplete = false; // Tracks if the flash‑and‑fade sequence ran once

  if (digitalRead(SWITCH_PIN4) == LOW) {
    if (!seqComplete) {
      // 1) Flash BLINKS_RING4 times at defined rate
      for (uint8_t i = 0; i < BLINKS_RING4; i++) {
        ring4.fill(ring4.Color(R4_BRIGHT_LVL, 0, 0)); // Full red ON
        ring4.show();
        delay(BLINK_ON_MS);
        ring4.clear();
        ring4.show();
        delay(BLINK_OFF_MS);
      }
      // 2) Bright‑Dim‑Bright pulse
      ring4.fill(ring4.Color(R4_BRIGHT_LVL, 0, 0)); // Bright red
      ring4.show();
      delay(DIM_DELAY_MS);
      ring4.fill(ring4.Color(R4_DIM_LVL, 0, 0));    // Dim red
      ring4.show();
      delay(DIM_DELAY_MS);
      ring4.fill(ring4.Color(R4_BRIGHT_LVL, 0, 0)); // Bright red again
      ring4.show();

      seqComplete = true; // Remember completion until button released
    } else {
      // Hold steady bright red after sequence finished
      ring4.fill(ring4.Color(R4_BRIGHT_LVL, 0, 0));
      ring4.show();
    }
  } else {
    ring4.clear();
    ring4.show();
    seqComplete = false; // Reset sequence when button released
  }

  // Shared frame delay for rings 1‑3 (ring 4 handles its own delays inside sequence)
  delay(SPEED_MS);
}
